require ( "util" ) 

-- storage.players = {}
-- player = storage.players[player_index] = {corpses={}}



local Body = {}

--• table_size ( table )  → uint https://lua-api.factorio.com/latest/auxiliary/libraries.html#table_size






local corpseFilter = {filter = "name", name = "character-corpse"}

local function newCorpse (corpseEntity)
	local corpse = {
		entity = corpseEntity,
		line = nil,
	}

	return corpse
end



local function newStoragePlayer (player) 
	local surfaces = {}
	local player_index = player.index
	for surface_name, surface in pairs ( game.surfaces ) do
--		https://lua-api.factorio.com/latest/classes/LuaSurface.html#find_entities_filtered
		local corpseEntities = surface.find_entities_filtered(corpseFilter)
--		https://lua-api.factorio.com/latest/classes/LuaEntity.html#character_corpse_player_index
		local corpses = {}
		for _, corpseEntity in ipairs (corpseEntities) do
			local character_corpse_player_index = corpseEntity.character_corpse_player_index
			if character_corpse_player_index == player_index then
				local corpse = newCorpse (corpseEntity)
				table.insert (corpses, corpse)
			end

		end

		surfaces[surface_name] = {corpses = corpses}
		game.print ('added surface: '.. surface_name)
		game.print ('amount surfaces: '.. table_size (surfaces))
	end

	local storagePlayer = {
		surfaces = surfaces,
		index = player.index,
	}

	return storagePlayer
end

local function updateplayer (player)
--	https://lua-api.factorio.com/latest/classes/LuaPlayer.html
	local player_index = player.index
	game.print ('player_index '.. player_index)
	local storagePlayer = storage.players[player_index]
	if not storage.players[player_index] then 
		storage.players[player_index] = newStoragePlayer (player)
	end
end

-- Init / Migrate
function Body.on_init (  ) 
	game.print ('Body.on_init')
	storage.players = storage.players or {} -- new players table

	for player_index, player in pairs ( game.players ) do
		updateplayer (player)
	end
end

function Body.on_configuration_changed (  )
	game.print ('Body.on_configuration_changed')
	storage.players = storage.players or {} -- new players table
	for player_index, player in pairs ( game.players ) do
		updateplayer (player)
	end
end

local function newStorageSurface (player_index, surfaceName)
	local storagePlayer = storage.players[player_index]
	local storageSurface = {corpses = {}}
	storagePlayer.surfaces[surfaceName] = storageSurface
	return storageSurface
end

--function Body.on_player_died ( event ) 
--	local player_index = event.player_index
--	local player = game.players[player_index]

--	local storagePlayer = storage.players[player_index]
----	if not storagePlayer then
----		game.print ('no storagePlayer')
----	end

--	local surfaceName = player.surface.name
--	local storageSurface = storagePlayer[surfaceName]
--	if not storageSurface then
----		game.print ('no storageSurface')
--		storageSurface = newStorageSurface (player_index, surfaceName)
--	end

--	local position = player.position
--	local corpse = {
--		surface_name = player.surface.name, 

--	}
--	corpse.entity = player.surface.find_entity ( "character-corpse", player.position ) 

--	if ( corpse.entity and corpse.entity.valid ) then
--		table.insert ( storageSurface.corpses, corpse ) 
--	end
--end

local function addStoragePlayer (player_index)
	local storagePlayer = {surfaces = {}}
	storage.players[player_index] = storagePlayer
	return storagePlayer
end

local function addStorageSurface (surfaceName, player_index)

	local storagePlayer = storage.players[player_index]
	local storageSurface = {corpses = {}} 

	storagePlayer.surfaces[surfaceName] = storageSurface

	return storageSurface
end


local function addCorpse (surfaceName, player_index, corpseEnitity)
	------
	game.print ('addCorpse')
	local storagePlayer = storage.players[player_index]
	if not storagePlayer then
		game.print ('addStoragePlayer')
		storagePlayer = addStoragePlayer (player_index)
	end

	------
	local storageSurface = storagePlayer.surfaces[surfaceName]
	if not storageSurface then
		game.print ('addStorageSurface')
		storageSurface = addStorageSurface (surfaceName, player_index)
	end

	local corpse = newCorpse (corpseEnitity)
	table.insert (storageSurface.corpses, corpse)
	game.print ('amount corpses on '..surfaceName..': '.. #storageSurface.corpses)

end

function Body.on_post_entity_died ( event)
	local prototype = event.prototype -- The entity prototype of the entity that died.
	local prototypeType = prototype.type

	if prototypeType == 'character' then
		local position = event.position
		local corpses = event.corpses
		local surface_index = event.surface_index
		local surface_name = game.surfaces[surface_index].name
		game.print ('surfaceindex: '..surface_index..'; surface name: '..surface_name)

		for i, corpse in pairs (corpses) do
			game.print ('corpse.type: '..corpse.type)
			game.print ('corpse.prototype.type: '..corpse.prototype.type)
			local player_index = corpse.character_corpse_player_index
			if player_index then

				addCorpse (surface_name, player_index, corpse)
			end
		end
	end
end

function Body.on_player_created(event)
	local player = game.players[event.player_index]
	local surface_name = player.surface.name

	storage.players = storage.players or {}
	storage.players[event.player_index] = {
		player_index = event.player_index,
		surfaces = {
--			[surface_name] = { corpses = {} }
		},
		activeSurface = surface_name
	}
end

function Body.on_tick_event  ( event ) 
	if ( event.tick%120 == 0 ) then
		for player_index in pairs ( storage.players ) do
			Body.redraw_lines ( player_index ) 
		end
	end
end


-----------------------------------------------------

-- Rendering functions
function Body.reduce_color ( c, f ) 
	f = f or 0.1
	return {r =  ( c.r or 1 ) *f, g =  ( c.g or 1 ) *f, b =  ( c.b or 1 ) *f, a =  ( c.a or 1 ) *f}
end

function Body.distance_sort ( a, b ) 
	return a.distance>b.distance
end




local function updateDistances (player_position, corpses)
	for corpse_index, corpse in ipairs ( corpses ) do
		local entity = corpse.entity
		corpse.distance = util.distance ( player_position, entity.position )
	end
end

local function distantSortFunction ( a, b ) 
	return a.distance>b.distance
end

local function updateLines (player, corpses)
	local amount = #corpses
	for i = 1, #corpses do
		local corpse = corpses[i]
		local factor =  ( i/amount ) 
		local lineWidth = math.max ( factor, 0.2 ) *1.5
		local lineColor = Body.reduce_color ( player.color, math.max ( factor, 0.2 )  )
		if ( corpse.line and corpse.line.valid ) then
			corpse.line.width = lineWidth
			corpse.line.color = lineColor 
		else
			-- new rendering.line
			corpse.line = rendering.draw_line {
				color = lineColor, 
				width = lineWidth, 
				from = player.character, 
				to = corpse.entity, 
				surface = player.surface, 
				players = {player.name}, 
				only_in_alt_mode = true
			}
		end
	end
end


function Body.redraw_lines ( player_index ) 
	local player = game.players[player_index]
	local player_position = player.position

	if ( not player.character ) then return end

	local storagePlayer = storage.players[player_index]

	if storagePlayer then

		local surface = player.surface
		local surface_name = surface.name

		local storageSurface = storagePlayer.surfaces[surface_name]

		if storageSurface then
			local corpses = storageSurface.corpses
			updateDistances (player_position, corpses)
			table.sort ( storageSurface, distantSortFunction ) 
			updateLines (player, corpses)

		end

	end
end

local function destroyRendering  ( renderingLine ) 
	-- Factorio 2.0
	-- https://lua-api.factorio.com/latest/classes/LuaRenderObject.html#valid
	if renderingLine and renderingLine.valid then
--		rendering.destroy ( renderingLine ) 
	end
end

-- Destroy storage corpse object by surface
function Body.clean_surface ( surface ) 
	for index, storagePlayer in pairs ( storage.players ) do
		for i, corpse in pairs ( storagePlayer.corpses ) do
			if ( corpse.surface_name == surface.name ) then
				destroyRendering  ( corpse.line ) 
				storagePlayer.corpses[i] = nil
			end
		end
	end
end

-- Destroy storage corpse object by entity
function Body.destroy_corpse ( entity )
	for player_index, storagePlayer in pairs ( storage.players ) do
		for surfaceName, storageSurface in pairs ( storagePlayer.surfaces ) do
			for i, corpse in pairs ( storageSurface.corpses ) do
				if ( corpse.entity == entity ) then
					destroyRendering  ( corpse.line ) 
					table.remove (storageSurface.corpses, i)
					return
				end
			end
		end
	end
end

function Body.on_player_respawned ( event ) 
	if not storage.players[event.player_index] then
		updateplayer (game.players[event.player_index])
	end
	game.print ('game surfaces')
	for surface_name, surface in pairs (game.surfaces) do
		game.print (surface_name .. ' ' .. surface.name)
	end

	game.print ('storage players surfaces')
	local player_index = event.player_index
	game.print ('player_index ' .. player_index)
	if storage.players[player_index] then
		game.print ('storage.players[player_index] exists')
	end

	if storage.players[player_index].surfaces then
		game.print ('storage.players[player_index].surfaces exists')
	end

	local surface = game.players[player_index].surface
	local surface_name = surface.name
	if surface_name then
		game.print ('surface_name '..surface_name)
	end

	if storage.players[player_index].surfaces[surface_name] then
		game.print ('storage.players[player_index].surfaces[surface_name] exists')
	end

	for surface_name, storageSurface in pairs (storage.players[event.player_index].surfaces) do
		game.print (surface_name)
	end
	Body.redraw_lines ( event.player_index ) 
end

function Body.on_player_changed_surface ( event ) 
	Body.redraw_lines ( event.player_index ) 
end

function Body.on_pre_surface_cleared ( event ) 
	Body.clean_surface ( game.surfaces[event.surface_index] ) 
end

function Body.on_pre_surface_deleted ( event ) 
	Body.clean_surface ( game.surfaces[event.surface_index] ) 

	for index, player in pairs ( game.players ) do
		if ( player.surface.index == event.surface_index ) then
--			Body.redraw_lines ( player.index ) 
		end
	end
end

function Body.on_character_corpse_expired ( event ) 
	Body.destroy_corpse ( event.corpse ) 
end

function Body.on_pre_player_mined_item ( event ) 
	Body.destroy_corpse ( event.entity ) 
end


----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------


script.on_init ( Body.on_init ) 

script.on_configuration_changed ( Body.on_configuration_changed ) 


---- events
script.on_event ( defines.events.on_player_created, Body.on_player_created ) 

--script.on_event ( defines.events.on_player_died, Body.on_player_died ) 

script.on_event ( defines.events.on_post_entity_died, Body.on_post_entity_died ) 

--script.on_event ( defines.events.on_tick, Body.on_tick_event ) 

script.on_event ( defines.events.on_pre_surface_cleared, Body.on_pre_surface_cleared ) 

script.on_event ( defines.events.on_player_changed_surface, Body.on_player_changed_surface ) 

script.on_event ( defines.events.on_player_respawned, Body.on_player_respawned ) 

script.on_event ( defines.events.on_pre_surface_deleted, Body.on_pre_surface_deleted ) 

script.on_event ( defines.events.on_character_corpse_expired, Body.on_character_corpse_expired ) 

script.on_event ( defines.events.on_pre_player_mined_item, Body.on_pre_player_mined_item, {{filter = "name", name = "character-corpse"}} )


----------------------------------------------------------------------
----------------------------------------------------------------------
----------------------------------------------------------------------

